/*
 * $Id: //depot/prod/capstan/RCapstan_Hudson.2/src/java/com/netapp/dlm/repository/hibernate/SessionWrapper.java#2 $
 * 
 * Copyright (c) 2005 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.dlm.repository.hibernate;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import net.sf.hibernate.FlushMode;
import net.sf.hibernate.HibernateException;
import net.sf.hibernate.Query;
import net.sf.hibernate.Session;
import net.sf.hibernate.StaleObjectStateException;

/**
 * This is a wrapper for the hibernate Session class for the purpose of honoring the hibernate LGPL.
 * 
 * @author taylor
 * @version $Revision: #2 $ $Date: 2017/02/21 $
 */
public class SessionWrapper {
    /**
     * The realy hinernate session that we are wrapping.
     */
    private final Session hibernateSession;
    private final LobCleanUpInterceptor interceptor;

    /**
     * Set the real hinernate session.
     * @param session The real hibernate session.
     */
    public SessionWrapper(Session session, LobCleanUpInterceptor interceptor) {
        this.hibernateSession = session;
        this.interceptor = interceptor;
    }
    
    public LobCleanUpInterceptor getInterceptor() {
        return this.interceptor;
    }

    public void setFlushMode(FlushMode mode) {
        this.hibernateSession.setFlushMode(mode);
    }

    public void evict(java.lang.Object object) throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.evict(object);
        } 
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    public void clear() {
        try {
            this.hibernateSession.clear();
        }
        catch (Exception e) {
            //do nothing for exception
        }
    }


    /**
     * Wrapper for the hibernate session save method.
     * @param arg0 Object to save.
     * @throws HibernateExceptionWrapper
     */
    public void save(Object arg0) throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.save(arg0);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    /**
     * Wrapper for the hibernate session delete method.
     * @param obj Object to delete.
     * @throws HibernateExceptionWrapper
     */
    public void delete(Object obj) throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.delete(obj);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    /**
     * Wrapper for the hibernate session saveOrUpdate method.
     * 
     * @param arg0 Object to save or update.
     * @throws HibernateExceptionWrapper
     */
    public void saveOrUpdate(Object arg0) throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.saveOrUpdate(arg0);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    /**
     * Wrapper for the hibernate session update method.
     * 
     * @param arg0 Repository object to update.
     * @throws HibernateExceptionWrapper
     */
    public void update(Object arg0) throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.update(arg0);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    public void refresh(Object arg0) throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.refresh(arg0);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    /**
     * Create a wrapped hibernate query.
     * @param queryString Query string.
     * @return HibernateQueryWrapper 
     * @throws HibernateExceptionWrapper
     */
    public QueryWrapper createQuery(String queryString) throws HibernateExceptionWrapper {
        try {
            return new QueryWrapper(this.hibernateSession.createQuery(queryString));
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }
    
    /**
     * Create a wrapped hibernate query with pagination.
     * 
     * @param queryString
     *            Query string.
     * @return HibernateQueryWrapper
     * @throws HibernateExceptionWrapper
     */
    public QueryWrapper createQueryWithPagination(String queryString, Integer offset, Integer limit)
        throws HibernateExceptionWrapper {
        try {
            Query query = this.hibernateSession.createQuery(queryString);
            query.setFirstResult(offset);
            query.setMaxResults(limit);
            return new QueryWrapper(query);
        }

        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    /**
     * Wrapper for hibernate session find method.
     * @param queryString Query string.
     * @return List of results.
     */
    public List find(String queryString) throws HibernateExceptionWrapper {
        try {
            return this.hibernateSession.find(queryString);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    public void flush() throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.flush();
        }
        catch (StaleObjectStateException e) {
            throw new StaleObjectStateExceptionWrapper(e);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    public void rollback() throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.connection().rollback();
        }
        catch (SQLException e) {
            throw new HibernateExceptionWrapper(e);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    public void commit() throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.connection().commit();
        }
        catch (SQLException e) {
            throw new HibernateExceptionWrapper(e);
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    public void close() throws HibernateExceptionWrapper {
        try {
            this.hibernateSession.close();
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    public Connection connection() throws HibernateExceptionWrapper {
        try {
            return this.hibernateSession.connection();
        }
        catch (HibernateException e) {
            throw new HibernateExceptionWrapper(e);
        }
    }

    public boolean isOpen() {
        return this.hibernateSession.isOpen();
    }

    public boolean isConnected() {
        return this.hibernateSession.isConnected();
    }

    public boolean contains(Object o) {
        return this.hibernateSession.contains(o);
    }
}
